//
//  PointManager.swift
//  SwiftVector
//
//  Created by Crazy凡 on 16/9/8.
//  Copyright © 2016年 Crazy凡. All rights reserved.
//

import UIKit
let frequency:CGFloat = 22
let relation:CGFloat = 200
let k:CGFloat = 10

class PointManager: NSObject {
    
    static let shareManager = PointManager()
    
    var points:Array<Point>
    
    private let screenSize = UIScreen.mainScreen().bounds.size
    private var timer:NSTimer
    
    override init() {
        self.points = Array();
        self.timer = NSTimer.init();
        super.init();
        self.timer = NSTimer.scheduledTimerWithTimeInterval(Double(1/frequency), target: self, selector: #selector(PointManager.movePoint), userInfo: nil, repeats: true)
        self.pauseMove()
    }
    
    func addPoints(number:Int) {
        for _ in 0..<number {
            let point = Point.init(frame: CGRectZero)
            point.s = Vector.randomVector(screenSize)
            point.setPosition()
            points.append(point)
        }
    }
    
    func addPointToView(view:UIView) {
        for point in points {
            view .addSubview(point)
        }
    }
    
    @objc private func movePoint() {
        for pa in points {
            for pb in points {
                if pa != pb {
                    let x = Vector.vectorFromVectorAToVectorB(pa.s, vectorB: pb.s)
                    //                    let delta:CGFloat = x.length() - relation
                    pa.f.add(x.normalize(k * (x.length() - relation)))
                }
            }
            pa.a = pa.f;
            pa.v.add(pa.a.multipy(1/frequency))
            pa.s.add(pa.v.multipy(1/frequency))
            pa.setPosition()
        }
    }
    
    func getCurrentPositionPoint(position:CGPoint) -> Point? {
        for point in points {
            if Vector.vectorWithVector(point.s).minus(Vector.vectorWithPoint(position)).length() < CGRectGetWidth(point.frame) {
                return point
            }
        }
        return nil
    }
    
    func movePointToNewPosition(point:Point, position:CGPoint) {
        point.moveToPosition(position)
    }
    
    func moveAllWithVector(vector:Vector) {
        for point in points {
            point.moveWithVector(vector)
        }
    }
    
    func startMove(){
        timer.fireDate = NSDate.init()
    }
    
    func pauseMove(){
        timer.fireDate = NSDate.distantFuture()
    }
}


